/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package net.sf.xradar;

import java.io.File;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.maven.project.MavenProject;
import org.apache.maven.reporting.AbstractMavenReport;
import org.apache.maven.reporting.MavenReportException;
import org.codehaus.doxia.site.renderer.SiteRenderer;
import org.codehaus.plexus.resource.ResourceManager;


/**
 * Base class for XRadar report.
 * @author Nicolas Dordet
 */
public abstract class AbstractXRadarMojo extends AbstractMavenReport{

	/**
	 * The output directory for the final HTML report. Note that this parameter is only evaluated if the goal is run
	 * directly from the command line or during the default lifecycle. If the goal is run indirectly as part of a site
	 * generation, the output directory configured in the Maven Site Plugin is used instead.
	 * 
	 * @parameter expression="${project.reporting.outputDirectory}"
	 * @required
	 */
	protected File outputDirectory;


	/**
	 * @see org.apache.maven.reporting.AbstractMavenReport#getOutputDirectory()
	 */
	protected String getOutputDirectory()
	{
		return outputDirectory.getAbsolutePath();
	}

	 /**
     * Whether to build an aggregated report at the root, or build individual reports.
     *
     * @parameter expression="${aggregate}" default-value="false"
     * @since 1.2
     */
    protected boolean aggregate;
    
    /**
     * The directories containing the sources to be compiled.
     *
     * @parameter expression="${project.compileSourceRoots}"
     * @required
     * @readonly
     * @since 1.2
     */
    private List compileSourceRoots;
    
    /**
     * The directories containing the test-sources to be compiled.
     *
     * @parameter expression="${project.testCompileSourceRoots}"
     * @required
     * @readonly
     * @since 1.2
     */
    private List testSourceRoots;



	/**
	 * Version of the project.
	 * 
	 * @parameter expression="${project.version}" default-value="${project.version}"
	 * @required
	 */
	protected String projectVersion;

	/**
	 * @component
	 * @required
	 * @readonly
	 */
	protected ResourceManager locator;

	/**
	 *
	 * @parameter expression="${xradar.config.location}"
	 *            default-value="config/xradar-config-default.xml"
	 */
	protected String configLocation;

	/**
	 *
	 * @parameter expression="${xradar.config.location}"
	 *            default-value="config/default-release.properties"
	 */
	protected String propertiesLocation;

	/**
	 * Root directory of the project.
	 *
	 * @parameter expression="${basedir}"
	 */
	protected String projectDirectory;


	/**
	 * The Maven project which is using this plugin.
	 * 
	 * @parameter expression="${project}"
	 * @required
	 */
	protected MavenProject project;

	/**
	 * Location of XRadar configuration file.
	 * 
	 * @parameter expression="${xradarConfig}" default-value="${basedir}/src/main/resources/xradar/radar-config.xml"
	 * @required
	 */
	protected File xradarConfig;

	/**
	 * The xradar's properties. 
	 * 
	 * @parameter expression="${xradarProperties}"
	 * default-value="${basedir}/src/main/resources/xradar/release${project.version}.properties"
	 * @required
	 */
	protected File xradarProperties;

	/**
	 * The radar target for docs.
	 * 
	 * @parameter expression="${target}"
	 * default-value="${project.build.directory}/site/xradar/"
	 */
	protected File xradarTarget;
	//default-value="${basedir}/xradar/release${project.version}/docs/"

	/**
	 * The debug mode.
	 * 
	 * @parameter expression="${debug}" 
	 * default-value="false"
	 */
	protected boolean debug;

	/**
	 * The debug mode.
	 * 
	 * @parameter expression="${debugMojo}" 
	 * default-value="false"
	 */
	protected boolean debugMojo;

	static final String FILE_SEP = System.getProperty("file.separator");
	
	public boolean isExternalReport() {
		return true;
	}

	@Override
	public boolean canGenerateReport() {
		// TODO Auto-generated method stub
		return true;
	}

	@Override
	protected void executeReport(Locale arg0) throws MavenReportException {
		// TODO Auto-generated method stub

	}

	/**
	 * Site rendering component for generating the HTML report.
	 *
	 * @component
	 */
	private SiteRenderer siteRenderer;

	/**
	 * @see org.apache.maven.reporting.AbstractMavenReport#getSiteRenderer()
	 */
	protected SiteRenderer getSiteRenderer() {

		return siteRenderer;
	}


	public abstract String getDescription(Locale arg0);

	public abstract String getName(Locale arg0);

	public abstract String getOutputName();

	protected ResourceBundle getBundle( Locale locale )
	{         
		return ResourceBundle.getBundle( "xradar-report", locale, this.getClass().getClassLoader() );     
	}

	public String getProjectVersion() {
		return projectVersion;
	}

	public void setProjectVersion(String projectVersion) {
		this.projectVersion = projectVersion;
	}

	public ResourceManager getLocator() {
		return locator;
	}

	public void setLocator(ResourceManager locator) {
		this.locator = locator;
	}

	public String getConfigLocation() {
		return configLocation;
	}

	public void setConfigLocation(String configLocation) {
		this.configLocation = configLocation;
	}

	public String getPropertiesLocation() {
		return propertiesLocation;
	}

	public void setPropertiesLocation(String propertiesLocation) {
		this.propertiesLocation = propertiesLocation;
	}

	public String getProjectDirectory() {
		return projectDirectory;
	}

	public void setProjectDirectory(String projectDirectory) {
		this.projectDirectory = projectDirectory;
	}

	public File getXradarConfig() {
		return xradarConfig;
	}

	public void setXradarConfig(File xradarConfig) {
		this.xradarConfig = xradarConfig;
	}

	public File getXradarProperties() {
		return xradarProperties;
	}

	public void setXradarProperties(File xradarProperties) {
		this.xradarProperties = xradarProperties;
	}

	public File getXradarTarget() {
		return xradarTarget;
	}

	public void setXradarTarget(File xradarTarget) {
		this.xradarTarget = xradarTarget;
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public boolean isDebugMojo() {
		return debugMojo;
	}

	public void setDebugMojo(boolean debugMojo) {
		this.debugMojo = debugMojo;
	}
	
	protected MavenProject getProject() {
		return project;
	}
	public void setProject(MavenProject project) {
		this.project = project;
	}

	public boolean isAggregate() {
		return aggregate;
	}

}
