/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package net.sf.xradar;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import net.sf.xradar.util.Copy;

import org.apache.maven.reporting.MavenReportException;
import org.codehaus.plexus.resource.loader.FileResourceCreationException;
import org.codehaus.plexus.resource.loader.FileResourceLoader;
import org.codehaus.plexus.resource.loader.ResourceNotFoundException;
import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.config.ReleaseData;
import org.sourceforge.xradar.config.ReleaseDataLoader;
import org.sourceforge.xradar.dynamics.Dynamics;
import org.sourceforge.xradar.dynamics.Input;
import org.sourceforge.xradar.util.XRadarUtils;

/**
 * <p>Goal which creates an XRadar dynamic report.</p>
 *
 * @author Nicolas Dordet
 * @goal dynamic 
 */
public class XRadarDynamicMojo extends AbstractXRadarMojo
{

	public String getDescription( Locale locale )
	{         
		return getBundle( locale ).getString( "report.xradar-dynamic-report.description" );     
	}

	public String getName( Locale locale )
	{         
		return getBundle( locale ).getString( "report.xradar-dynamic-report.name" );     
	}

	public String getOutputName()
	{         
		return "xradar/dynamics/docs/index";     
	}

	/**
	 * Location of the output files.
	 * @parameter expression="${basedir}/xradar/dynamics/docs"
	 * @required
	 */
	private String outputDirectory;
	/*
	 * Included element
	 */
	private List<Input> inputs = new ArrayList<Input>(0);	
	/*
	 * Internal fields
	 */
	private Dynamics dynamicsEngine;
	private boolean dynamic = false;
	private boolean staticsConfigured = true;


	public void addInput(Input input)
	{
		if ( this.inputs == null )
		{
			this.inputs = new ArrayList<Input>();
		}
		this.inputs.add(input);
	}



	@Override
	protected void executeReport(Locale locale) throws MavenReportException 
	{

//TODO : add a condition on xradar static (exist?) for pom project
		getLog().info("Actual Project version Number : " + projectVersion);
		getLog().info("xradarTarget.exists() : "+ xradarTarget.exists());

		dynamic = true;
		getLog().info("Getting properties files informations ....");

		locator.addSearchPath( FileResourceLoader.ID, project.getFile().getParentFile().getAbsolutePath() );
		locator.addSearchPath( "url", "" );
		locator.setOutputDirectory(new File( project.getBuild().getDirectory() ));
		Thread.currentThread().setContextClassLoader( this.getClass().getClassLoader() );
		try {
			if (!xradarProperties.exists())
			{
				File xradarProperties = locator.getResourceAsFile( propertiesLocation);
				setXradarProperties(xradarProperties);

			}
			ReleaseData releaseData = new ReleaseDataLoader(XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(getXradarProperties().getAbsolutePath()))).load();

			Input in = new Input();
			in.setVersionId(releaseData.getSequencdeId());
			getLog().info("VersionId : " + releaseData.getSequencdeId());
			in.setDaysSinceLast(Integer.valueOf(releaseData.getVersion()));
			//in.setStaticsRoot(projectDirectory+"/xradar/release"+project.getVersion()+"/docs");
			//getLog().info("StaticRoot : " + projectDirectory+"/xradar/release"+releaseData.getSystem()+"/docs");
			in.setStaticsRoot(XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(xradarTarget.getAbsolutePath())));
			getLog().info("StaticRoot : " + xradarTarget);
			in.setQualityReportPath(XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(in.getStaticsRoot()+"/xml/reports/")));
			in.setMasterFinal(XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(projectDirectory+"/xradar/dynamics/docs/xml/dynamics-master-final.xml")));
			in.setReportToMerge(XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(in.getStaticsRoot()+"/xml/radar_report_normalized.xml")));

			if(in.getVersionId()==1)
			{
				in.setPreviousReport(Dynamics.DEFAULT_BASE_MASTER_REPORT);
			}
			else
			{
				in.setPreviousReport(XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(projectDirectory+"/xradar/dynamics/docs/xml/dynamics-master-final.xml")));
			}
			addInput(in);

		} catch (ResourceNotFoundException e) {
			throw new MavenReportException("Unable to configure Xradar, a file is missing ",e);
		} catch (FileResourceCreationException e) {
			throw new MavenReportException("Unable to configure Xradar, a resource creation failed ",e);

		}

		if(dynamic)
		{

			getLog().info("Dynamic process starting .......");

			validateDynamic();
			dynamicsInitialization();
			try 
			{
				for ( Input input : this.inputs )
				{
					// Version specific settings
					dynamicsEngine.setSequenceId(String.valueOf(input.getVersionId()));
					dynamicsEngine.setDaysSinceLast(String.valueOf(input.getDaysSinceLast()));
					dynamicsEngine.setQualityReportPath(input.getQualityReportPath());
					dynamicsEngine.setMasterFinal(input.getMasterFinal());
					if ( isStaticsConfigured() )
					{
						dynamicsEngine.setStaticRoot(input.getStaticsRoot());
					}
					// Updating file from previous version to the current version
					getLog().info("Calling execute merge with " + input.getPreviousReport() + " and " + input.getReportToMerge());			
					this.dynamicsEngine.executeMerge(input.getPreviousReport(),input.getReportToMerge());
					// Post processing the previousVersion
					getLog().info("Calling post process with " +  input.getMasterFinal());
					this.dynamicsEngine.postProcess(input.getMasterFinal());
					// Adding static images
					getLog().info("Adding static images to website");
					this.dynamicsEngine.copyImagesToWebSite();
					// Generating the dynamics 'website'
					getLog().info("Calling dynamics style report with " +  input.getMasterFinal());
					this.dynamicsEngine.dynamicsStyleReport(input.getMasterFinal(),this.xradarTarget + Dynamics.EMPTY);
				}
			}
			catch (XRadarException e) 
			{
				throw new MavenReportException("problem occured while processing dynamic Engine",e);
			}
			getLog().info("copying files to target folder");
			new Copy(projectDirectory+"/xradar",xradarTarget.getAbsolutePath());
		}

	}

	/**
	 * <p>Init. method, initialize the dynamics engine from XRadar core.</p>
	 * @throws MavenReportException
	 */
	private void dynamicsInitialization() throws MavenReportException
	{
		try 
		{
			this.dynamicsEngine.init();
		} 
		catch (XRadarException e) 
		{
			throw new MavenReportException("",e);
		}
	}

	/**
	 * <p>Validate that the XML element value form the pom
	 * are correct and consistent.</p>
	 * 
	 * @throws MavenReportException
	 */
	private void validateDynamic() throws MavenReportException
	{
		// attribute validation
		if ( "".equals(this.getXradarConfig()) )
		{
			throw new MavenReportException("Config field is not optionnal.");
		}
		if ( "".equals(this.getXradarTarget()) )
		{
			throw new MavenReportException("Target field is not optionnal.");
		}
		// Validating 'input' reports
		if ( this.inputs.size() <= 0 )
		{
			throw new MavenReportException("At least one element <input> is required !");
		}
		// Logging settings
		getLog().info("Debug:" + this.debug);
		getLog().info("Target:" + this.xradarTarget);
		getLog().info("Config:" + this.xradarConfig);
		getLog().info("Is Statics configured ?" + this.isStaticsConfigured());

		// Validating each report
		for ( Input input : this.inputs )
		{
			validateDynamic(input);
		}
		// Setting the engine
		this.dynamicsEngine = new Dynamics();
		// Setting (and creating docs-home)
		this.dynamicsEngine.setDebug(this.debug);
		try {
			if (!xradarConfig.exists())
			{
				File xradarConfig = locator.getResourceAsFile( configLocation);
				setXradarConfig(xradarConfig);
			}
			this.dynamicsEngine.setConfig(this.getXradarConfig().getAbsolutePath());
		} catch (ResourceNotFoundException e) {
			throw new MavenReportException("Unable to configure Xradar, a file is missing ",e);
		} catch (FileResourceCreationException e) {
			throw new MavenReportException("Unable to configure Xradar, a resource creation failed ",e);
		}
		try 
		{
			this.dynamicsEngine.setDocsHome(projectDirectory+"/xradar/dynamics/docs/");
		}
		catch (XRadarException e) 
		{
			throw new MavenReportException("problem occured while setting docsHome",e);
		}
	}

	/**
	 * <p>Assert if the provided Input does complies with the need of a dynamic report
	 * generation.</p>
	 * 
	 * @param input
	 * @throws MavenReportException
	 */
	private void validateDynamic(Input input) throws MavenReportException 
	{
		String mssg = "Configure input with versionId" + input.getVersionId();
		// Previous version filename not setted
		if ( input.getPreviousReport() == null || "".equals(input.getPreviousReport()) )
		{
			// VersionId is 1, using default file
			if ( input.getVersionId() == 1 )
			{
				input.setPreviousReport(Dynamics.DEFAULT_BASE_MASTER_REPORT);
			}
			else // this is an error...
			{
				throw new MavenReportException("PreviousReport field is not optionnal." + mssg);			
			}
		}
		if ( input.getReportToMerge() == null || "".equals(input.getReportToMerge()))
		{
			throw new MavenReportException("Report-to-merge field is not optionnal." + mssg );
		}
		if ( input.getQualityReportPath() == null && "".equals(input.getQualityReportPath() ) )
		{
			throw new MavenReportException("qualityReportPath field is not optionnal."+ mssg );
		}
		if ( "".equals(input.getDaysSinceLast()) )
		{
			input.setDaysSinceLast(0);
		}
		if ( "".equals(input.getVersionId()) )
		{
			input.setVersionId(1);
		}
		if ( input.getMasterFinal() == null || "".equals(input.getMasterFinal()) )
		{
			throw new MavenReportException("masterFinal field is not optionnal." + mssg );
		}
		if ( this.isStaticsConfigured() )
		{
			if ( "".equals(input.getStaticsRoot()) )
			{
				throw new MavenReportException("Statics root is required if staticConfigured is set to true." + mssg );
			}
			//else
			//{
			//TODO: Check that directory exists and is readable ! 
			//TODO: Also check for an index.html file
			//}
		}
		getLog().info("Following input is valid:" + input.toString());
	}	

	/**
	 * @return the staticsConfigured
	 */
	public boolean isStaticsConfigured() {
		return staticsConfigured;
	}

	/**
	 * @param staticsConfigured the staticsConfigured to set
	 */
	public void setStaticsConfigured(boolean staticsConfigured) {
		this.staticsConfigured = staticsConfigured;
	}

	public String getOutputDirectory() {
		return outputDirectory;
	}


}
