/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package net.sf.xradar;

import java.io.File;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import net.sf.xradar.aggregate.PrepareProjectDirectories;
import net.sf.xradar.util.FileFinder;
import net.sf.xradar.util.Filter;
import net.sf.xradar.util.XRadarMerger;
import static net.sf.xradar.executor.MojoExecutor.artifactId;
import static net.sf.xradar.executor.MojoExecutor.configuration;
import static net.sf.xradar.executor.MojoExecutor.element;
import static net.sf.xradar.executor.MojoExecutor.executeMojo;
import static net.sf.xradar.executor.MojoExecutor.executionEnvironment;
import static net.sf.xradar.executor.MojoExecutor.goal;
import static net.sf.xradar.executor.MojoExecutor.groupId;
import static net.sf.xradar.executor.MojoExecutor.name;
import static net.sf.xradar.executor.MojoExecutor.plugin;
import static net.sf.xradar.executor.MojoExecutor.version;

import org.apache.maven.BuildFailureException;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.cli.ConsoleDownloadMonitor;
import org.apache.maven.embedder.MavenEmbedder;
import org.apache.maven.embedder.MavenEmbedderException;
import org.apache.maven.embedder.PlexusLoggerAdapter;
import org.apache.maven.execution.MavenSession;
import org.apache.maven.lifecycle.LifecycleExecutionException;
import org.apache.maven.monitor.event.DefaultEventMonitor;
import org.apache.maven.monitor.event.EventMonitor;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.PluginManager;
import org.apache.maven.project.DuplicateProjectException;
import org.apache.maven.reporting.MavenReportException;
import org.apache.maven.wagon.events.TransferListener;

import org.codehaus.plexus.resource.loader.FileResourceCreationException;
import org.codehaus.plexus.resource.loader.FileResourceLoader;
import org.codehaus.plexus.resource.loader.ResourceNotFoundException;
import org.codehaus.plexus.util.dag.CycleDetectedException;

import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.config.ReleaseData;
import org.sourceforge.xradar.config.ReleaseDataLoader;
import org.sourceforge.xradar.statics.MergeFileBackboneCreator;
import org.sourceforge.xradar.statics.MergeProduct;
import org.sourceforge.xradar.statics.Param;
import org.sourceforge.xradar.statics.Report;
import org.sourceforge.xradar.statics.Statics;
import org.sourceforge.xradar.statics.renderers.XslHtmlWebsite;
import org.sourceforge.xradar.util.XRadarUtils;


/**
 * Goal which creates an XRadar static report.
 * @author Nicolas Dordet
 * @goal audit
 */
public class XRadarMojo extends AbstractXRadarMojo
{

	/**
	 * The Maven PluginManager Object
	 *
	 * @component
	 * @required
	 */
	protected PluginManager pluginManager;

	public String getDescription( Locale locale )
	{         
		return getBundle( locale ).getString( "report.xradar-static-report.description" );     
	}

	public String getName( Locale locale )
	{         
		return getBundle( locale ).getString( "report.xradar-static-report.name" );     
	}


	public String getOutputName()
	{     
		return "xradar/index";
	}


	/*
	 * Fields accessible through getter/setters
	 */

	/**
	 * Directory containing the class files.
	 *
	 * @parameter expression="${project.build.outputDirectory}"
	 * @required
	 */
	private File classDirectory;

	/**
	 * The radar javadoc.
	 * 
	 * @parameter expression="${javadocEnabled}"
	 * default-value="true" 
	 */
	private String javadocEnabled;
	//FIXME : Boolean?


	/**
	 * Location of the generated findBugs xml report.
	 *
	 * @parameter expression="${project.build.directory}/findbugs.xml"
	 * 
	 */
	private File reportFindBugs;

	/**
	 * Location of the generated jdepend's xml report.
	 *
	 * @parameter expression="${project.build.directory}/jdepend-report.xml"
	 * 
	 */
	private File reportJdepend;    

	/**
	 * Location of the generated ckjm's xml report.
	 *
	 * @parameter expression="${project.build.directory}/ckjm-report.xml"
	 * 
	 */
	private File reportCkjm;

	/**
	 * Location of the generated cpd's xml report.
	 *
	 * @parameter expression="${project.build.directory}/cpd.xml"
	 * 
	 */
	private File reportCpd;

	/**
	 * Location of the generated pmd's xml report.
	 *
	 * @parameter expression="${project.build.directory}/pmd.xml"
	 * 
	 */
	private File reportPmd;

	/**
	 * Location of the generated checkstyle's xml report.
	 *
	 * @parameter expression="${project.build.directory}/checkstyle-result.xml"
	 * 
	 */
	private File reportCheckstyle;

	/**
	 * Location of the generated javancss's xml report.
	 *
	 * @parameter expression="${project.build.directory}/javancss-raw-report.xml"
	 * 
	 */
	private File reportJavancss;

	/**
	 * Location of the generated corbetura's xml report.
	 *
	 * @parameter expression="${project.build.directory}/site/cobertura/coverage.xml"
	 * 
	 */
	private File reportCobertura;

	/**
	 * Location of the generated corbetura's xml report.
	 *
	 * @parameter expression="${project.build.directory}/site/emma/coverage.xml"
	 * 
	 */
	private File reportEmma;

	/**
	 * Location of the generated jUnit xml report.
	 *
	 * @parameter expression="${project.build.directory}/surefire-reports/junit.xml"
	 * 
	 */
	private File reportJUnit;

	/**
	 * Location of the generated jUnit xml report.
	 *
	 * @parameter expression="${project.build.directory}/surefire-reports/"
	 * 
	 */
	private File directoryJUnit;

	/**
	 * Location of the generated xref directory.
	 *
	 * @parameter expression="${project.build.directory}/site/xref"
	 * 
	 */
	private File directoryJxr; //FIXME : if changed it will not be changed on xradar, since it need relative path

	/**
	 * Location of the generated javadoc directory.
	 *
	 * @parameter expression="${project.build.directory}/site/apidocs" 
	 * 
	 */
	private File directoryJavadoc;


	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableFindBugs}" 
	 * default-value="true"
	 */
	private boolean enableFindBugs;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enablePmd}" 
	 * default-value="true"
	 */
	private boolean enablePmd;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableCheckstyle}" 
	 * default-value="true"
	 */
	private boolean enableCheckstyle;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableCkjm}" 
	 * default-value="true"
	 */
	private boolean enableCkjm;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableJavancss}" 
	 * default-value="true"
	 */
	private boolean enableJavancss;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableEmma}"
	 * default-value="true"
	 */
	private boolean enableEmma;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableCobertura}"
	 * default-value="true"
	 */
	private boolean enableCobertura;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableJavadoc}" 
	 * default-value="true"
	 */
	private boolean enableJavadoc;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableJxr}" 
	 * default-value="true"
	 */
	private boolean enableJxr;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableJdepend}" 
	 * default-value="true"
	 */
	private boolean enableJdepend;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableSurfire}" 
	 * default-value="true"
	 */
	private boolean enableSurefire;

	/**
	 * enable/disable a plugin. (automatic mode)
	 * 
	 * @parameter expression="${enableCpd}" 
	 * default-value="true"
	 */
	//private boolean enableCpd;

	/**
	 *
	 * @parameter expression="${xradar.auto}"
	 *            default-value="true"
	 */
	private boolean autoRadar;

	/**
	 * The current maven session
	 * @parameter expression="${session}"
	 * @readonly
	 */
	private MavenSession session;

	/**
	 * Contains the full list of projects in the reactor.
	 *
	 * @parameter expression="${reactorProjects}"
	 * @readonly
	 */
	protected List reactorProjects;


	private final static String PLUGIN_JDEPEND = "jdepend-maven-plugin";
	private final static String PLUGIN_PMD = "maven-pmd-plugin";
	private final static String PLUGIN_CHECKSTYLE = "maven-checkstyle-plugin";
	private final static String PLUGIN_JXR = "maven-jxr-plugin";
	private final static String PLUGIN_JAVADOC = "maven-javadoc-plugin";
	private final static String PLUGIN_CKJM = "maven-ckjm-plugin";
	private final static String PLUGIN_JAVANCSS = "javancss-maven-plugin";
	private final static String PLUGIN_COBERTURA = "cobertura-maven-plugin";
	private final static String PLUGIN_FINDBUGS = "findbugs-maven-plugin";
	private final static String PLUGIN_EMMA = "emma-maven-plugin";
	private final static String PLUGIN_SUREFIRE = "maven-surefire-plugin";


	private static final String MAVEN_GROUP_ID = "org.apache.maven.plugins";
	private static final String CODEHAUS_GROUP_ID = "org.codehaus.mojo";

	/*
	 * Internal fields
	 */
	private XslHtmlWebsite staticEngine;

	private List<Report> reports;

	public void addReport(Report report)
	{
		if ( this.reports == null )
		{
			this.reports = new ArrayList<Report>();
		}
		this.reports.add(report);
	}

	/**
	 * This method check the plugins declared in the pom
	 * of the project to be audited and return the list of the reports plugins.
	 * @return pluginsReportList 
	 */
	public List<String> checkReportPlugins()
	{
		List<String> pluginsReportList = new ArrayList<String>();
		Set artifacts = project.getReportArtifacts();
		for (Iterator artifactIterator = artifacts.iterator(); artifactIterator.hasNext();) 
		{
			Artifact artifact = (Artifact) artifactIterator.next();
			pluginsReportList.add(artifact.getArtifactId());
			//FIXME : when a a plugin report tool is declared in a profile.
		}
		if(debugMojo)
		{
			getLog().info("List of Artifact found in the pom : ");
			for(int indexPlugin = 0; indexPlugin < pluginsReportList.size(); indexPlugin++)
			{
				getLog().info("ArtifactId from pom : "  + pluginsReportList.get(indexPlugin).toString());
			}

		}
		return pluginsReportList;
	}

	/**
	 * check if the classes are presents
	 * if there are in the right directory, run jdepend
	 * if not, exit.
	 * @param classDirectory
	 * @throws MavenReportException 
	 * @throws XRadarException 
	 */
	public void runJDepend(File classDirectory) throws MavenReportException
	{

		try {
			MergeFileBackboneCreator.createBackBoneFile(classDirectory,reportJdepend);
		} catch (XRadarException e) {
			throw new MavenReportException("failed to run jDepend",e);
		}

	}

	private List<MavenPlugin> reportPluginSuppList = new ArrayList<MavenPlugin>();

	/**
	 * method witch initialize the maven plugins
	 */
	public void init()
	{
		reportPluginSuppList.add(buildPlugin(CODEHAUS_GROUP_ID, PLUGIN_JDEPEND, "2.0-beta-2", "generate", enableJdepend, reportJdepend));
		reportPluginSuppList.add(buildPlugin(MAVEN_GROUP_ID,PLUGIN_PMD, "2.4", "pmd", enablePmd, reportPmd));
		reportPluginSuppList.add(buildPlugin(MAVEN_GROUP_ID, PLUGIN_CHECKSTYLE, "2.2","checkstyle", enableCheckstyle, reportCheckstyle));
		reportPluginSuppList.add(buildPlugin(CODEHAUS_GROUP_ID, PLUGIN_JAVANCSS, "2.0-beta-2", "report",enableJavancss, reportJavancss));
		reportPluginSuppList.add(buildPlugin("net.sourceforge.ckjm", PLUGIN_CKJM, "0.4", "ckjm", enableCkjm, reportCkjm));
		reportPluginSuppList.add(buildPlugin(MAVEN_GROUP_ID, PLUGIN_JXR, "2.1", "jxr", enableJxr, directoryJxr));
		reportPluginSuppList.add(buildPlugin(MAVEN_GROUP_ID, PLUGIN_JAVADOC, "2.5", "javadoc",enableJavadoc, directoryJavadoc));
		reportPluginSuppList.add(buildPlugin(CODEHAUS_GROUP_ID, PLUGIN_EMMA, "1.0-alpha-1","emma", enableEmma, reportEmma));
		reportPluginSuppList.add(buildPlugin(CODEHAUS_GROUP_ID,PLUGIN_FINDBUGS,"2.0.1","findbugs",enableFindBugs,reportFindBugs));
		reportPluginSuppList.add(buildPlugin(CODEHAUS_GROUP_ID,PLUGIN_COBERTURA,"2.3","cobertura",enableCobertura,reportCobertura));
		//TODO: check that : http://docs.codehaus.org/display/SONAR/Frequently+Asked+Questions#FrequentlyAskedQuestions-CorberturaexceptiononLinuxSystemwhileaccessingthecobertura.serfile
		// cobertura.setProperties((Properties) new Properties().setProperty("cobertura.report.format", "xml"));
		reportPluginSuppList.add(buildPlugin(MAVEN_GROUP_ID,PLUGIN_SUREFIRE,"2.4.3","test",enableSurefire,reportJUnit));
	}

	/**
	 * <p>An handy method to create a MavenPlugin instance. Help factorize code./<p>
	 * @param groupId
	 * @param artifactId
	 * @param version
	 * @param goal
	 * @param state
	 * @param report
	 * @return the plugin
	 */
	private MavenPlugin buildPlugin(String groupId, String artifactId, String version, String goal, boolean state, File report) 
	{
		MavenPlugin plugin = new MavenPlugin();
		plugin.setGroupId(groupId);
		plugin.setArtifactId(artifactId);
		plugin.setVersion(version);
		plugin.setGoal(goal);
		plugin.setEnabled(state);
		plugin.setReportFile(report);
		return plugin;
	}

	/**
	 * @throws XRadarException 
	 * @see org.apache.maven.reporting.AbstractMavenReport#executeReport( java.util.Locale )
	 */
	public void executeReport(Locale locale) throws MavenReportException
	{

		getLog().info("project.getName()"+ project.getName());
		getLog().info("project packaging : "+ project.getPackaging());

		if (isAggregate())
		{
			getLog().info("aggregate : "+ isAggregate());
			
			if(project.getModules().size() > 0 
					&& project.isExecutionRoot() 
					&& project.getCompileSourceRoots().size() > 0 
					&& !"pom".equals(project.getPackaging()))
			{
				getLog().info("project.isExecutionRoot()"+ project.isExecutionRoot());
				run();
			}

			//if there are submodules, aggregate projects on parent pom project
			if(project.getModules().size() > 0 )
			{
				//TODO : replace uggly +"/"+ ...
				File parentSrcDirectory = new File(project.getBuild().getDirectory() + FILE_SEP + "xradar" + FILE_SEP + "generated-classes" + FILE_SEP+ "sources");
				File parentClassDirectory = new File(project.getBuild().getDirectory() + FILE_SEP+ "xradar" + FILE_SEP +"generated-classes"+ FILE_SEP+ "classes");

				if (!parentSrcDirectory.exists())
				{
					parentSrcDirectory.mkdirs();
				}
				if (!parentClassDirectory.exists())
				{
					parentClassDirectory.mkdirs();
				}

				PrepareProjectDirectories assistant = new PrepareProjectDirectories();

				assistant.copyChildDirectory( reactorProjects ,  parentSrcDirectory.getAbsolutePath(), "src");
				assistant.copyChildDirectory( reactorProjects ,  parentClassDirectory.getAbsolutePath(), "class");
				assistant.copyChildDirectory( reactorProjects ,  parentSrcDirectory.getAbsolutePath(), "srcTest");
				assistant.copyChildDirectory( reactorProjects ,  parentClassDirectory.getAbsolutePath(), "classTest");

				project.getBuild().setSourceDirectory(parentSrcDirectory.getAbsolutePath()+FILE_SEP+"src/main/java");
				project.getBuild().setOutputDirectory(parentClassDirectory.getAbsolutePath()+FILE_SEP+"classes");
				project.getBuild().setTestSourceDirectory(parentSrcDirectory.getAbsolutePath()+FILE_SEP+"src/test/java");
				project.getBuild().setTestOutputDirectory(parentClassDirectory.getAbsolutePath()+FILE_SEP+"test-classes");
				//FIXME : emma and cobertura didn't works on parent aggregated files
				// since the project is not a "java" project (ex : pom project without src/main/java directory) 
				// http://svn.codehaus.org/mojo/tags/cobertura-maven-plugin-2.3/src/main/java/org/codehaus/mojo/cobertura/CoberturaInstrumentMojo.java
				// ArtifactHandler artifactHandler = project.getArtifact().getArtifactHandler();
				//    if ( !"java".equals( artifactHandler.getLanguage() ) )
				//    {
				//        getLog().info(
				//            "Not executing cobertura:instrument as the project is not a Java classpath-capable package" );
				//    }


				//File file = new File(parentClassDirectory.getAbsolutePath()+"/"+"classes");
				runJDepend(parentClassDirectory);

				run();


			}
			else if ("pom".equals(project.getPackaging()) && project.getModules().size()<= 0)
			{
				getLog().info("skipping project"+ project.getName() + "of type pom without submodules" );
			}
			else
			{
				run();
			}
		}
		else if (!"pom".equals(project.getPackaging()))
		{

			run();
		}


	}
	public void run()throws MavenReportException
	{
		boolean isJdependPresent = false;
		boolean isMavenEmbedderActive = true;
		init();

		//initializing lists to have :
		//- the plugins declared in the pom
		//- the plugins supported by xradar
		//- the plugins missing
		List<String> reportPluginSupportedAndUsed = new ArrayList<String>();
		List<String> reportPluginsList = new ArrayList<String>();
		reportPluginsList = checkReportPlugins();

		getLog().info("List of the plugins declared in the Pom of this project and supported by xradar :");

		for(int pluginIndex = 0 ; pluginIndex < reportPluginsList.size() ; pluginIndex++)
		{
			for (int pluginSupportedIndex = 0 ; pluginSupportedIndex < reportPluginSuppList.size() ; pluginSupportedIndex++)
			{
				if( reportPluginSuppList.get(pluginSupportedIndex).getArtifactId().compareTo(reportPluginsList.get(pluginIndex)) == 0)
				{
					getLog().info("Plugins supported and used  : " + reportPluginsList.get(pluginIndex));
					reportPluginSupportedAndUsed.add(reportPluginsList.get(pluginIndex));
					reportPluginSuppList.remove(pluginSupportedIndex);

					break;
				}
			}
		}
		if(debugMojo)
		{
			getLog().info("List of Artifact initialised after removing declared ones : ");
			for(int indexPlug = 0; indexPlug < reportPluginSuppList.size(); indexPlug++)
			{
				getLog().info("ArtifactId found : "  + reportPluginSuppList.get(indexPlug).getArtifactId());
				getLog().info("plugin isEnabled : "  + reportPluginSuppList.get(indexPlug).isEnabled());
			}
		}

		if(autoRadar)
		{
			MavenEmbedder embedder = new MavenEmbedder();
			TransferListener transferListener = new ConsoleDownloadMonitor();
			embedder.setLogger(new XRadarMavenEmbedderLogger(getLog()));
			EventMonitor eventMonitor = new DefaultEventMonitor(new PlexusLoggerAdapter(embedder.getLogger()));

			for(int pluginMissingIndex = 0 ; pluginMissingIndex < reportPluginSuppList.size() ; pluginMissingIndex++)
			{
				isMavenEmbedderActive = true;
				getLog().info("\n" + "Plugins Missing : " + reportPluginSuppList.get(pluginMissingIndex).getArtifactId());

				//JDepend have to be executed whatever append.
				if (PLUGIN_JDEPEND.equals(reportPluginSuppList.get(pluginMissingIndex).getArtifactId()) )
				{
					isJdependPresent = false; // have to put false : problem with profile and multimodule
				}
				//check that the plugin is enabled and if the report not already exist.
				else if(reportPluginSuppList.get(pluginMissingIndex).isEnabled() && !reportPluginSuppList.get(pluginMissingIndex).getReportFile().exists())
				{
					getLog().info("Embedding "+ reportPluginSuppList.get(pluginMissingIndex).getArtifactId() 
							+" with default's plugin configuration");

					Properties properties = new Properties();
					properties.setProperty("xradar.properties", "xradar");
					List<String> goals = new ArrayList<String>();

					//TODO find a better way to do that
					// when pmd has been found, have to call the goal cpd "manually".
					if(PLUGIN_PMD.equals(reportPluginSuppList.get(pluginMissingIndex).getArtifactId()))
					{
						goals.add(reportPluginSuppList.get(pluginMissingIndex).getFullGoal());
						goals.add("org.apache.maven.plugins:maven-pmd-plugin:2.2:cpd");

					}
					// Cobertura need a specific configuration 
					else if(PLUGIN_COBERTURA.equals(reportPluginSuppList.get(pluginMissingIndex).getArtifactId()))
					{
						goals.add(reportPluginSuppList.get(pluginMissingIndex).getFullGoal());
						properties.setProperty("cobertura.report.format", "xml");

					}
					else if(PLUGIN_SUREFIRE.equals(reportPluginSuppList.get(pluginMissingIndex).getArtifactId()))
					{
						goals.add(reportPluginSuppList.get(pluginMissingIndex).getFullGoal());
						properties.setProperty("maven.test.failure.ignore", "true");

					}
					else if(PLUGIN_JXR.equals(reportPluginSuppList.get(pluginMissingIndex).getArtifactId()))
					{
						/*goals.add(reportPluginSuppList.get(pluginMissingIndex).getFullGoal());
							properties.setProperty("destDir", outputDirectory.getAbsolutePath() + "/xref");
							properties.setProperty("outputDirectory", outputDirectory.getAbsolutePath());*/

						//Plugin jxr had problem working with multi-modules project.
						//when executing with the MojoExecutor, it work well.
						//So mavenEmbber is disabled for this one
						try {
							getLog().info("executing mojo JXR");
							getLog().info("full goal : " + reportPluginSuppList.get(pluginMissingIndex).getFullGoal());
							executeMojo(
									plugin(
											groupId(reportPluginSuppList.get(pluginMissingIndex).getGroupId()),
											artifactId(reportPluginSuppList.get(pluginMissingIndex).getArtifactId()),
											version(reportPluginSuppList.get(pluginMissingIndex).getVersion())
									),
									goal(reportPluginSuppList.get(pluginMissingIndex).getGoal()),
									configuration(
											element(name("destDir"), outputDirectory.getAbsolutePath() + "/xref"),
											element(name("outputDirectory"), outputDirectory.getAbsolutePath())
									),
									executionEnvironment(
											project,
											session,
											pluginManager
									)
							);
							isMavenEmbedderActive = false;
						} 
						catch (MojoExecutionException e1) 
						{
							throw new MavenReportException("Unable to launch goal",e1);
						}


					}
					else if(PLUGIN_FINDBUGS.equals(reportPluginSuppList.get(pluginMissingIndex).getArtifactId()))
					{

						//TODO try with mavenEmbedder + properties
						try {
							getLog().info("executing mojo Findbugs");
							getLog().info("full goal : " + reportPluginSuppList.get(pluginMissingIndex).getFullGoal());
							executeMojo(
									plugin(
											groupId(reportPluginSuppList.get(pluginMissingIndex).getGroupId()),
											artifactId(reportPluginSuppList.get(pluginMissingIndex).getArtifactId()),
											version(reportPluginSuppList.get(pluginMissingIndex).getVersion())
									),
									goal(reportPluginSuppList.get(pluginMissingIndex).getGoal()),
									configuration(
											element(name("xmlOutput"), "true")//,
											//element(name("findbugsXmlOutput"),"true")
									),
									executionEnvironment(
											project,
											session,
											pluginManager
									)
							);
							isMavenEmbedderActive = false;
						} 
						catch (MojoExecutionException e1) 
						{
							throw new MavenReportException("Unable to launch goal",e1);
						}


					}
					//the default execution
					else
					{
						goals.add(reportPluginSuppList.get(pluginMissingIndex).getFullGoal());

					}
					//embedding maven
					if (isMavenEmbedderActive)
					{
						try {	

							embedder.setClassLoader(session.getContainer().getContainerRealm().getClassLoader());
							embedder.start();
							embedder.execute(project, goals, eventMonitor, transferListener, properties, project.getBasedir());
							embedder.stop();

						} catch (CycleDetectedException e) {
							throw new MavenReportException("maven embedder error :",e);
						} catch (LifecycleExecutionException e) {
							throw new MavenReportException("maven embedder error :",e);
						} catch (BuildFailureException e) {
							throw new MavenReportException("maven embedder error :",e);
						} catch (DuplicateProjectException e) {
							throw new MavenReportException("maven embedder error :",e);
						} catch (MavenEmbedderException e) {
							throw new MavenReportException("maven embedder error :",e);
						}
					}
				}
				else
				{
					getLog().info(reportPluginSuppList.get(pluginMissingIndex).getArtifactId() 
							+" report file already exist or has been disabled" + "\n");
				}


			}
			reportPluginSuppList.removeAll(reportPluginSuppList);
		}

		getLog().info("-------------------------------------------------------------------------");

		if(!isJdependPresent)
		{
			getLog().info("JDepend plugin not found");
			getLog().info("Using JDepend as an API");
			runJDepend(classDirectory);
		}

		//FIXME : uggly fix to merge junit reports :
		if (directoryJUnit.exists())
		{
			ArrayList<File> jUnitReportsList = SeekJunitReports(directoryJUnit);//TODO : find a better way to do that,
																				//then remove FileFinder.java Filter.java and FilterDir.java

			if(!reportJUnit.exists())
			{
				XRadarMerger xm = new XRadarMerger();
				xm.mergeJunit(jUnitReportsList, reportJUnit.getAbsolutePath());
			}
		}

		//add report in order to merge them
		addReport( new Report("jdepend",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportJdepend.getAbsolutePath()))));
		addReport(new Report("ckjm",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportCkjm.getAbsolutePath()))));
		addReport(new Report("cpd",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportCpd.getAbsolutePath()))));
		addReport(new Report("pmd",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportPmd.getAbsolutePath()))));
		addReport(new Report("checkstyle",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportCheckstyle.getAbsolutePath()))));
		addReport(new Report("javancss",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportJavancss.getAbsolutePath()))));
		addReport(new Report("findbugs",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportFindBugs.getAbsolutePath()))));
		addReport(new Report("junit",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportJUnit.getAbsolutePath()))));
		addReport(new Report("emma",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportEmma.getAbsolutePath()))));
		addReport(new Report("cobertura",XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(reportCobertura.getAbsolutePath()))));
		validate();
		doMerge();
	}

	public ArrayList<File> SeekJunitReports (File directoryJUnit)
	{
		ArrayList<File> filesFound = new FileFinder().findFilesFrom(directoryJUnit.getAbsolutePath(), new Filter() , false); 
		return filesFound;
	}

	public void doMerge() throws MavenReportException
	{
		try
		{
			// Merging the report
			String mergedReport = this.staticEngine.executeMerge();
			if (debug)
			{
				getLog().info("XML merging result is :\n" + mergedReport);
			}
			getLog().info("Merging data from tool's reports.");
			// Preparing the post processing
			//if xradarConfig or xradarProperties not found, call default one.
			locator.addSearchPath( FileResourceLoader.ID, project.getFile().getParentFile().getAbsolutePath() );
			locator.addSearchPath( "url", "" );
			locator.setOutputDirectory(new File( project.getBuild().getDirectory() ));
			Thread.currentThread().setContextClassLoader( this.getClass().getClassLoader() );
			if (!xradarConfig.exists())
			{
				File xradarConfig = locator.getResourceAsFile( configLocation);
				setXradarConfig(xradarConfig);
			}

			Param radarConfig = new Param(Statics.XRADAR_CONFIG_FILE,XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(xradarConfig.getAbsolutePath())));
			if (!xradarProperties.exists())
			{
				File xradarProperties = locator.getResourceAsFile( propertiesLocation);
				setXradarProperties(xradarProperties);

			}
			ReleaseData releaseData = new ReleaseDataLoader(XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName(xradarProperties.getAbsolutePath()))).load();
			// Post processing
			getLog().info("Post processing merged files.");
			MergeProduct products = this.staticEngine.executePostProcessing(mergedReport, releaseData, radarConfig);
			// Generating the website
			Map<String,Param> params = new HashMap<String,Param>();
			// FIXME:normalize and remove disk driver name
			params.put(Statics.JAVADOC_ENABLED,new Param(Statics.JAVADOC_ENABLED,this.getJavadocEnabled()));
			params.put(Statics.JAVADOC_ROOT,new Param(Statics.JAVADOC_ROOT,this.directoryJavadoc.toURL().toString()));
			params.put(Statics.JAVA2HTML_ROOT,new Param(Statics.JAVA2HTML_ROOT,"../xref"));
			//FIXME jxr link is relative and need to be adjusted. Why it's not the same transformation as JAVADOC_ROOT ?
			params.put(Statics.XRADAR_CONFIG_FILE,new Param(Statics.XRADAR_CONFIG_FILE,XRadarUtils.removeDiskNameIfRunningOnWindows(XRadarUtils.normalizeFileName((this.getXradarConfig().getAbsolutePath())))));
			params.put(Statics.LOG,new Param(Statics.LOG,String.valueOf(this.debug)));
			getLog().info("Generating static analysis website.");
			// Still buggy, only works for images generation !
			this.staticEngine.generateHtmlReport(products, params);
			
			// Copying required resources
			this.staticEngine.copyStaticResourcesToWebSite(this.getXradarTarget()+FILE_SEP,this.getXradarTarget() + "/xml/api");
		}
		catch (XRadarException e)
		{
			throw new MavenReportException("Unable to configure Xradar",e);
		} 
		catch (ResourceNotFoundException e) 
		{
			throw new MavenReportException("ressource not found ",e);
		} 
		catch (FileResourceCreationException e) 
		{
			throw new MavenReportException("Unable to create resource ",e);
		} 
		catch (MalformedURLException e) 
		{
			throw new MavenReportException("Unable to convert file path to URL ",e);
		}
	}

	/**
	 * Checks if settings are proper, such as:
	 * <ul>
	 * 		<li>set docs home directory based on target attribute ( TODO: check if exist)</li>
	 * 		<li>set debug level based on debug attribute</li>
	 * 		<li>TODO:Report file exists and can be read</li>
	 * 		<li>If not defined load default value for order-id and xslt file</li>
	 * 		<li>TODO:Xslt file and can be accessed through</li>
	 * 		<li>... more ?</li>
	 * </ul>
	 * @throws MojoExecutionException 
	 * @throws MalformedURLException
	 *
	 */
	private void validate() throws MavenReportException
	{
		// attribute validation
		if ( "".equals(this.getXradarTarget()) | this.getXradarTarget()==null)
		{
			throw new MavenReportException("Target field is not optionnal.");
		}
		if ( "".equals(this.getXradarProperties()) | this.getXradarProperties()==null)
		{
			throw new MavenReportException("Properties field is not optionnal.");
		}
		if ( "".equals(this.getXradarConfig()) |this.getXradarConfig()==null)
		{
			throw new MavenReportException("Config field is not optionnal.");
		}
		if ( "".equals(this.directoryJxr.getAbsolutePath()) |this.getXradarConfig()==null)
		{
			throw new MavenReportException("Java2htmlRoot field is not optionnal.");
		}
		if ( "".equals(this.directoryJavadoc.getAbsolutePath()) && Boolean.valueOf(this.javadocEnabled) )
		{
			throw new MavenReportException("JavadocRoot field is not optionnal, if javadocEnabled is set to true.");
		}

		//TODO make a logger class
		//TODO some info are default and can be overriden... log the right ones
		// Logging settings
		if(debugMojo)
		{
			getLog().info("*************************************************");
			getLog().info("Configuration declared for the XRadar's mojo");
			getLog().info("*************************************************");
			getLog().info("Project parameters : ");
			getLog().info("................................................");
			getLog().info("projectVersion : " + projectVersion);
			getLog().info("projectName : " + project.getName());
			getLog().info("projectPackaging : " + project.getPackaging());
			getLog().info("projectDirectory : " + projectDirectory);
			getLog().info("outputDirectory : "+ outputDirectory);
			getLog().info("................................................");
			getLog().info("Xradar parameters: ");
			getLog().info("................................................");
			getLog().info("autoRadar : " + autoRadar);
			getLog().info("xradarConfig : " + xradarConfig);
			getLog().info("xradarProperties : " + xradarProperties);
			getLog().info("aggregate : "  + isAggregate());
			getLog().info("classDirectory : " + classDirectory);
			getLog().info("xradarTarget : " + xradarTarget);
			getLog().info("debug: " + debug);
			getLog().info("................................................");
			getLog().info("Reports parameters : ");
			getLog().info("................................................");
			getLog().info("reportJdepend : "  + reportJdepend);
			getLog().info("reportCkjm : "  + reportCkjm);
			getLog().info("reportPmd : "  + reportPmd);
			getLog().info("reportCpd : "  + reportCpd);
			getLog().info("reportCheckstyle : "  + reportCheckstyle);
			getLog().info("reportJavancss : "  + reportJavancss);
			getLog().info("reportCobertura : "  + reportCobertura);
			getLog().info("reportEmma : "  + reportEmma);
			getLog().info("reportJunit : "  + reportJUnit);
			getLog().info("*************************************************");
			getLog().info("end of configuration");
			getLog().info("*************************************************");
		}
		// Setting the engine
		this.staticEngine = new XslHtmlWebsite();
		this.staticEngine.setDebug(this.debug);
		// Setting (and creating docs-home)
		try
		{
			this.staticEngine.setDocsHome(this.getXradarTarget()+FILE_SEP);
		}
		catch (XRadarException e)
		{
			throw new MavenReportException("execption",e);
		}
		// Addind and validating the reports ( validation delegated to the engine)
		for ( Report report : reports )
		{
			// Logging the added report data
			getLog().info("Report:"+ report.getType());
			getLog().info("->Input file:" + report.getFile());
			// Adding the report
			try
			{
				if ( ! this.staticEngine.addReport(report) )
				{
					getLog().info("Report " + report.getType() + " was not added, because " + report.getFile() + " does not exist or is not readable.");
				}
			}
			catch (XRadarException e)
			{
				throw new MavenReportException("exception",e);
			}
		}
	}

	//getters/setters


	public String getJavadocEnabled() {
		return javadocEnabled;
	}
	public void setJavadocEnabled(String javadocEnabled) {
		this.javadocEnabled = javadocEnabled;
	}

	
}
